;; Generated package description from delight.el  -*- no-byte-compile: t -*-
(define-package "delight" "1.7" "A dimmer switch for your lighter text" '((cl-lib "0.5") (nadvice "0.3")) :commit "a763ec1e5c2987efea3ce2ee6d9c979f56ab6528" :authors '(("Phil Sainty" . "psainty@orcon.net.nz")) :maintainer '("Phil Sainty" . "psainty@orcon.net.nz") :keywords '("convenience") :url "https://savannah.nongnu.org/projects/delight")
